--[[-------------------------------------------------------------------------
  Copyright (c) 2007, Trond A Ekseth
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of Maneki nor the names of its contributors
        may be used to endorse or promote products derived from this
        software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

--[[ 

	Updated by Martin Karer / Sezz, 2008-2010
	
	------------------------------
	
	Changelog:
	2010-01-08:
		[+] Disenchant Button
		[+] Test Roll - see Test() and TestCancel()
		
--]]


local _G = getfenv(0)
local print = function(a) ChatFrame1:AddMessage("|cff33ff99Maneki:|r "..tostring(a)) end
local error = function(...) print("|cffff0000Error:|r" .. string.format(...)) end

local TimerOnUpdate = function(self)
	if(self:IsVisible()) then
		local left = GetLootRollTimeLeft(self:GetParent():GetID())
		local min, max = self:GetMinMaxValues()
		if((left < min) or (left > max)) then left = min end

		self:SetValue(left, max)
	end
end

-- TODO: Format...
local TimeOnUpdate = function(self)
	local cur = GetLootRollTimeLeft(self:GetID())
	self.time:SetText(string.format("%1.f", (self.xmax-(self.max-cur))/1000))
end

local GeneralOnLeave = function()
	GameTooltip:Hide()
end

local PassOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(PASS)
end

local PassOnClick = function(self)
	RollOnLoot(self:GetParent():GetID(), 0)
	GameTooltip:Hide()
end

local GreedOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(GREED)
end

local GreedOnClick = function(self)
	RollOnLoot(self:GetParent():GetID(), 2)
	GameTooltip:Hide()
end

local NeedOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(NEED)
end

local NeedOnClick = function(self)
	RollOnLoot(self:GetParent():GetID(), 1)
	GameTooltip:Hide()
end

local DisenchantOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(ROLL_DISENCHANT)
end

local DisenchantOnClick = function(self)
	RollOnLoot(self:GetParent():GetID(), 3)
	GameTooltip:Hide()
end

local HoverOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetLootRollItem(self:GetParent():GetID())
--	CursorUpdate()
end

local HoverOnLeave = function(self)
	GameTooltip:Hide()
	ResetCursor()
end

local HoverOnClick = function(self)
	if(IsControlKeyDown()) then
		DressUpItemLink(GetLootRollItemLink(self:GetParent():GetID()))
	elseif(IsShiftKeyDown()) then
		ChatEdit_InsertLink(GetLootRollItemLink(self:GetParent():GetID()))
	end
end

local OnFade = function(self)
	self:Hide()
end

local addon = CreateFrame"Frame"
local style
local cache
local styles = {}
local furui = {}
local subTypes = {
	["IconHover"] = function(self)
		self:SetScript("OnClick", HoverOnClick)
		self:SetScript("OnEnter", HoverOnEnter)
		self:SetScript("OnLeave", HoverOnLeave)
	end,
	-- StatusBar
	["Timer"] = function(self)
		self:SetScript("OnUpdate", TimerOnUpdate)
	end,
	-- FontString
	["Time"] = function(self, owner)
		if(not owner.Timer) then
			owner:SetScript("OnUpdate", TimeOnUpdate)
		end
	end,
	-- Button
	["Pass"] = function(self)
		self:SetScript("OnClick", PassOnClick)
		self:SetScript("OnEnter", PassOnEnter)
		self:SetScript("OnLeave", GeneralOnLeave)
	end,
	-- Button
	["Greed"] = function(self)
		self:SetScript("OnClick", GreedOnClick)
		self:SetScript("OnEnter", GreedOnEnter)
		self:SetScript("OnLeave", GeneralOnLeave)
	end,
	-- Button
	["Disenchant"] = function(self)
		self:SetScript("OnClick", DisenchantOnClick)
		self:SetScript("OnEnter", DisenchantOnEnter)
		self:SetScript("OnLeave", GeneralOnLeave)
	end,
	-- Button
	["Need"] = function(self)
		self:SetScript("OnClick", NeedOnClick)
		self:SetScript("OnEnter", NeedOnEnter)
		self:SetScript("OnLeave", GeneralOnLeave)
	end,
}

local getFrame = function(id)
	for _, frame in pairs(cache) do
		if(not frame.shown) then
			frame:SetID(id)
			return frame
		end
	end
end

function addon:RegisterStyle(name, func)
	if(type(name) ~= "string") then return error("Bad argument #1 to 'RegisterStyle' (string expected, got %s)", type(name)) end
	if(type(func) ~= "function") then return error("Bad argument #2 to 'RegisterStyle' (function expected, got %s)", type(func)) end
	if(styles[name]) then return error("Style [%s] already registered.", name) end
	styles[name] = func
	
	if(not style) then
		self:SetActiveStyle(name)

		UIParent:UnregisterEvent"START_LOOT_ROLL"
		self:RegisterEvent"START_LOOT_ROLL"
		self:RegisterEvent"CANCEL_LOOT_ROLL"
	end
end

function addon:SetActiveStyle(name)
	if(type(name) ~= "string") then return error("Bad argument #1 to 'SetActiveStyle' (string expected, got %s", type(name)) end
	if(not styles[name]) then return error("Style [%s] does not exist.", name) end
	if(style) then furui[style] = cache end

	style = name
	cache = furui[name]

	if(not cache) then
		cache = {}
		for i=1, NUM_GROUP_LOOT_FRAMES do
			local object = styles[style]()
			
			-- Quick and Dirty Positioning Hack
			local p1, parent, p2, x, y
			if(not p1) then
				-- Grow-Up
				if (i > 1) then
					object:SetPoint("TOP", "UIParent", "BOTTOM", 0, 265 + ((i - 1) * (object:GetHeight() + 15)))
				else
					object:SetPoint("TOP", "UIParent", "BOTTOM", 0, 265)
				end
			end

			for subType, subObject in pairs(object) do
				if(subTypes[subType]) then
					subTypes[subType](subObject, object)
				end
			end

			cache[i] = object
		end
	end
end

function addon:ToggleButton(button, enabled)
	if (enabled ~= nil and enabled) then
		GroupLootFrame_EnableLootButton(button)
	else
		GroupLootFrame_DisableLootButton(button)
	end
end

addon.START_LOOT_ROLL = function(self, event, id, duration, test)
	if(not style) then return end

	local texture, name, count, quality, bindOnPickUp, canNeed, canGreed, canDisenchant
	if (test ~= nil) then
		name, _, quality, _, _, _, _, count, _, texture, _ = GetItemInfo(id)
		bindOnPickUp = false
		canNeed = false
		canGreed = false
		canDisenchant = false
		
		print('Test Roll: '..name..' '..quality..' '..count..' '..texture)
	else
		texture, name, count, quality, bindOnPickUp, canNeed, canGreed, canDisenchant = GetLootRollItemInfo(id);

	end

	local frame = getFrame(id)

	local r, g, b = GetItemQualityColor(quality)
	if(frame.Icon) then frame.Icon:SetTexture(texture) end
	if(frame.Item) then
		frame.Item:SetText(name)
		frame.Item:SetTextColor(r, g, b)
	end
	if(frame.BackdropQualityColor) then
		frame:SetBackdropColor(r, g, b)
	end
	if(frame.Timer) then
		frame.Timer:SetMinMaxValues(0, duration)
		frame.Timer:SetStatusBarColor(r, g, b, select(4, frame:GetBackdropColor()))
	end
	
	addon:ToggleButton(frame.Need, canNeed)
	addon:ToggleButton(frame.Greed, canGreed)
	addon:ToggleButton(frame.Disenchant, canDisenchant)

	UIFrameFadeIn(frame, .3, 0, 1)
	frame.shown = true
	frame:Show()
end

addon.CANCEL_LOOT_ROLL = function(self, event, id)
	for _, frame in pairs(cache) do
		if(frame:GetID() == id) then
			UIFrameFadeOut(frame, .3, 1, 0)
			frame.fadeInfo.finishedFunc = OnFade
			frame.fadeInfo.finishedArg1 = frame
			frame.shown = nil
		end
	end
end

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, event, ...)
end)

function addon:Test()
	addon.START_LOOT_ROLL(nil, nil, 21213, 60000, true);
end

function addon:TestCancel()
	print('Test Roll CANCELED!')
	addon.CANCEL_LOOT_ROLL(nil, nil, 21213);
end

_G.Maneki = addon
